/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung;

import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.uda.interpreter.daten.dav.MengenObjekt;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.KonfigurationsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * die Repräsentation einer Mengenanmeldung.<br>
 * Eine Mengenanmeldung ist eine Anmeldung auf die Änderung des Inhaltes einer
 * dynamischen Menge. Die ist nur als Empfänger möglich.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class MengenAnmeldungsEintrag implements AnmeldungsEintrag {

	private static final Debug LOGGER = Debug.getLogger();
	/**
	 * die Menge für die die Anmeldung erfolgen soll.
	 */
	private final MengenObjekt menge;

	/**
	 * Konstruktor.
	 *
	 * @param menge
	 *            die Menge für die die Anmeldung erfolgen soll.
	 */
	public MengenAnmeldungsEintrag(final MengenObjekt menge) {
		this.menge = menge;
	}

	/**
	 * {@inheritDoc}. Die Funktion führt die Abmeldung für die gegebene Menge
	 * beim Datenverteiler aus. Handelt es sich bei der übergebenen nicht um
	 * eine dynamisch änderbare Menge wird ein KonfigurationsFehler erzeugt.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.AnmeldungsEintrag#abmelden(de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.AnmeldeTyp,
	 *      de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.Anmeldung)
	 */
	@Override
	public void abmelden(final AnmeldeTyp typ, final Anmeldung anmeldung) {
		final ObjectSet set = menge.getObjekt();
		if (!(set instanceof MutableSet)) {
			throw new KonfigurationsFehler(UdaFehlerSubtyp.MENGE);
		}
		try {
			((MutableSet) set).removeChangeListener(anmeldung);
		} catch (final DataNotSubscribedException e) {
			LOGGER.error(e.getLocalizedMessage());
			// nicht angemeldete Listener können beim Abmelden ignoriert werden
		}

	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion führt die Anmeldung für die gegebene Menge beim
	 * Datenverteiler aus. Handelt es sich bei der übergebenen nicht um eine
	 * dynamisch änderbare Menge wird ein KonfigurationsFehler erzeugt.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.AnmeldungsEintrag#anmelden(de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.AnmeldeTyp,
	 *      de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.Anmeldung)
	 */
	@Override
	public void anmelden(final AnmeldeTyp typ, final Anmeldung anmeldung) {
		final ObjectSet set = menge.getObjekt();
		if (!(set instanceof MutableSet)) {
			throw new KonfigurationsFehler(UdaFehlerSubtyp.MENGE);
		}
		((MutableSet) set).addChangeListener(anmeldung);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Folgende Attribute können abgefragt werden:
	 * <ul>
	 * <li><b>menge:</b> das Mengenobjekt für die eine Anmeldung erfolgen soll.
	 * </li>
	 * </ul>
	 *
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object result = null;
		if ("menge".equals(name)) {
			result = menge;
		}
		if (result == null) {
			throw new ObjektZugriffsFehler(UdaFehlerSubtyp.ATTRIBUT);
		}
		return null;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion liefert den Wert true, wenn das übergebene Systemobjekt der
	 * anzumeldenden Menge entspricht. Die Parameter für die Attributgruppe und
	 * den Aspekt werden nicht berücksichtigt.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.AnmeldungsEintrag#matches(de.bsvrz.dav.daf.main.config.SystemObject,
	 *      de.bsvrz.dav.daf.main.config.AttributeGroup,
	 *      de.bsvrz.dav.daf.main.config.Aspect)
	 */
	@Override
	public boolean matches(final SystemObject sysObj, final AttributeGroup atg, final Aspect asp) {
		boolean result = false;
		if (menge.getObjekt().equals(sysObj)) {
			result = true;
		}
		return result;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Eine Änderung der Attribute einer Mengenanmeldung ist nicht möglich. Der
	 * Aufruf der Funktion erzeugt daher inm jedem Fall einen
	 * Objektzugriffsfehler.
	 *
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		throw new ObjektZugriffsFehler(UdaFehlerSubtyp.OBJEKT);
	}
}
