/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

import de.bsvrz.ibv.uda.interpreter.AbstractUdaHandler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArithmetischerFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.HandlerValidation;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * Handler arithmetische Ausdrücke.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ArithmetikHandler extends AbstractUdaHandler {

	/**
	 * Die Funktion liefert den Absolutbetrag für den übergebenen Wert.<br>
	 * Das Ergebnis ist eine Ganzzahl, wenn der übergebene Wert eine Ganzzahl
	 * ist und eine Fliesskommazahl, wenn eine Fliesskommazahl übergeben wurde.
	 *
	 * @param wert
	 *            der Wert
	 * @return der absolute Betrag
	 */
	private Number abs(final Number wert) {
		if (wert instanceof Long) {
			return Long.valueOf(Math.abs(wert.longValue()));
		}
		return Double.valueOf(Math.abs(wert.doubleValue()));
	}

	/**
	 * die Funktion ermittelt die Summe zweier übergebener Zahlenwert. Das
	 * Ergebnis ist eine Ganzzahl, wenn beide übergebenen Summanden Ganzzahlen
	 * sind, in allen anderen Fällen ist das Ergebnis eine Fließkommazahl.
	 *
	 * @param summand1
	 *            der erste Summand
	 * @param summand2
	 *            der zweite Summand
	 * @return die Summe
	 */
	private Number addition(final Number summand1, final Number summand2) {
		Number summe = summand1;

		if (summe instanceof Long) {
			if (summand2 instanceof Long) {
				summe = summe.longValue() + summand2.longValue();
			} else {
				summe = summe.longValue() + summand2.doubleValue();
			}
		} else if (summe instanceof Double) {
			if (summand2 instanceof Long) {
				summe = summe.doubleValue() + summand2.longValue();
			} else {
				summe = summe.doubleValue() + summand2.doubleValue();
			}
		}

		return summe;
	}

	/**
	 * Die Funktion liefert den Cosinuswert für den übergebenen Winkel im
	 * Gradmaß.<br>
	 * Das Ergebnis ist immer eine Fliesskommazahl.
	 *
	 * @param wert
	 *            der Winkel
	 * @return der Wert des Cosinus
	 */
	private Double cos(final Number wert) {
		return Double.valueOf(Math.cos(Math.toRadians(wert.doubleValue())));
	}

	/**
	 * Die Funktion dividiert die übergebenen Operanden. Das Ergebnis ist in
	 * jedem Fall eine Fließkommazahl.
	 *
	 * @param dividend
	 *            der Dividend
	 * @param divisor
	 *            der Divisor
	 * @return der Quotient
	 */
	private Double division(final Number dividend, final Number divisor) {
		if (divisor.longValue() == 0) {
			throw new ArithmetischerFehler(UdaFehlerSubtyp.DIVISION, "Division durch 0");
		}

		return dividend.doubleValue() / divisor.doubleValue();
	}

	/**
	 * Die Funktion liefert den Wert der Exponentialfunktion e hoch n für den
	 * übergebenen Wert.<br>
	 * Das Ergebnis ist immer eine Fliesskommazahl.
	 *
	 * @param exponent
	 *            der Wert
	 * @return der Exponentialwert
	 */
	private Double exp(final Number exponent) {
		return Double.valueOf(Math.exp(exponent.doubleValue()));
	}

	/**
	 * Die Funktion dividiert die übergebenen Operanden. Das Ergebnis ist in
	 * jedem Fall eine Ganzzahl. Bei Fließkommazahlen wird der gebrochene Anteil
	 * entfernt und danach dividiert.
	 *
	 * @param dividend
	 *            der Dividend
	 * @param divisor
	 *            der Divisor
	 * @return der Quotient
	 */
	private Long ganzzahlDivision(final Number dividend, final Number divisor) {
		if (divisor.longValue() == 0) {
			throw new ArithmetischerFehler(UdaFehlerSubtyp.DIVISION, "Division durch 0");
		}

		return dividend.longValue() / divisor.longValue();
	}

	/**
	 * Rest der Ganzzahldivision. Die Operation folgt der in JAVA verwendeten
	 * Vorgehensweise. Das Ergebnis der Operation ergibt sich als:
	 *
	 * <pre>
	 *                                      x % y = (int)(x - (int)(x/y)*y)
	 * </pre>
	 *
	 * Das Ergebnis ist eine Ganzzahl, wenn beide Operanden eine Ganzzahl sind,
	 * in allen anderen Fällen liefert die Operation eine Fließkommazahl.
	 *
	 * @param dividend
	 *            der Dividend
	 * @param divisor
	 *            der Divisor
	 * @return der Rest der Division
	 */
	private Number ganzzahlRest(final Number dividend, final Number divisor) {
		if (divisor.longValue() == 0) {
			throw new ArithmetischerFehler(UdaFehlerSubtyp.DIVISION, "Division durch 0");
		}

		Number rest = null;

		if (dividend instanceof Long) {
			if (divisor instanceof Long) {
				rest = dividend.longValue() % divisor.longValue();
			} else {
				rest = dividend.longValue() % divisor.doubleValue();
			}
		} else {
			rest = dividend.doubleValue() % divisor.doubleValue();
		}

		return rest;
	}

	/**
	 * die Funktion liefert den mit dem Index definierten Operand aus der
	 * übergebenen Liste der Operanden als Instanz des Typs Number.
	 *
	 * @param operanden
	 *            die Liste der Operanden
	 * @param index
	 *            der Index
	 * @return der Operand als Number
	 */
	private Number geNumberOperand(final List<Object> operanden, final int index) {
		Number result = null;
		if (operanden.size() > index) {
			final Object operand = operanden.get(index);
			if (operand instanceof Number) {
				if (operand instanceof Double) {
					result = (Number) operand;
				} else if (operand instanceof Float) {
					result = ((Number) operand).doubleValue();
				} else if (operand instanceof BigDecimal) {
					result = ((Number) operand).doubleValue();
				} else if (operand instanceof Long) {
					result = (Number) operand;
				} else if (operand instanceof AtomicInteger) {
					result = ((Number) operand).longValue();
				} else if (operand instanceof AtomicLong) {
					result = ((Number) operand).longValue();
				} else if (operand instanceof BigInteger) {
					result = ((Number) operand).longValue();
				} else if (operand instanceof Byte) {
					result = ((Number) operand).longValue();
				} else if (operand instanceof Integer) {
					result = ((Number) operand).longValue();
				} else if (operand instanceof Short) {
					result = ((Number) operand).longValue();
				}
			}
		}

		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @return die Liste der unterstützten Operatoren
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#getHandledOperators()
	 */
	@Override
	public Operator[] getHandledOperators() {
		return ArithmetikOperators.OPERATOREN;
	}

	/**
	 * Die Funktion vergleicht zwei Zahlenwerte miteinander. Es wird "wahr"
	 * geliefert, wenn beide Zahlen den gleichen Wert repräsentieren.
	 *
	 * @param wert1
	 *            der erste Wert
	 * @param wert2
	 *            der zweite Wert
	 * @return das vergleichsergebnis
	 */
	private LogischerWert gleich(final Number wert1, final Number wert2) {
		boolean vergleich = true;
		if ((wert1 instanceof Long) && (wert2 instanceof Long)) {
			vergleich = wert1.longValue() == wert2.longValue();
		} else {
			vergleich = wert1.doubleValue() == wert2.doubleValue();
		}

		return LogischerWert.valueOf(vergleich);
	}

	/**
	 * Die Funktion vergleicht zwei Zahlenwerte miteinander. Es wird "wahr"
	 * geliefert, wenn der erste Wert eine größere Zahl repräsentiert als der
	 * zweite.
	 *
	 * @param wert1
	 *            der erste Wert
	 * @param wert2
	 *            der zweite Wert
	 * @return das vergleichsergebnis
	 */
	private LogischerWert groesser(final Number wert1, final Number wert2) {
		boolean vergleich = true;
		if ((wert1 instanceof Long) && (wert2 instanceof Long)) {
			vergleich = wert1.longValue() > wert2.longValue();
		} else {
			vergleich = wert1.doubleValue() > wert2.doubleValue();
		}

		return LogischerWert.valueOf(vergleich);
	}

	/**
	 * Die Funktion vergleicht zwei Zahlenwerte miteinander. Es wird "wahr"
	 * geliefert, wenn der erste Wert eine größere oder gleich große Zahl
	 * repräsentiert wie der zweite.
	 *
	 * @param wert1
	 *            der erste Wert
	 * @param wert2
	 *            der zweite Wert
	 * @return das vergleichsergebnis
	 */
	private LogischerWert groesserGleich(final Number wert1, final Number wert2) {
		boolean vergleich = true;
		if ((wert1 instanceof Long) && (wert2 instanceof Long)) {
			vergleich = wert1.longValue() >= wert2.longValue();
		} else {
			vergleich = wert1.doubleValue() >= wert2.doubleValue();
		}

		return LogischerWert.valueOf(vergleich);
	}

	/**
	 * Die Funktion liefert den Wert eines in Grad übergebenen Winkels im
	 * Bogenmass.<br>
	 * Das Ergebnis ist immer eine Fliesskommazahl.
	 *
	 * @param winkel
	 *            der Winkel
	 * @return der Wert im Bogenmass
	 */
	private Double inBogenmass(final Number winkel) {
		return Double.valueOf(Math.toRadians(winkel.doubleValue()));
	}

	/**
	 * Die Funktion liefert den Wert eines im Bogenmass übergebenen Winkels in
	 * Grad.<br>
	 * Das Ergebnis ist immer eine Fliesskommazahl.
	 *
	 * @param winkel
	 *            der Winkel
	 * @return der Wert im Bogenmass
	 */
	private Double inGrad(final Number winkel) {
		return Double.valueOf(Math.toDegrees(winkel.doubleValue()));
	}

	/**
	 * Die Funktion vergleicht zwei Zahlenwerte miteinander. Es wird "wahr"
	 * geliefert, wenn der erste Wert eine kleinere Zahl repräsentiert als der
	 * zweite.
	 *
	 * @param wert1
	 *            der erste Wert
	 * @param wert2
	 *            der zweite Wert
	 * @return das vergleichsergebnis
	 */
	private LogischerWert kleiner(final Number wert1, final Number wert2) {
		boolean vergleich = true;
		if ((wert1 instanceof Long) && (wert2 instanceof Long)) {
			vergleich = wert1.longValue() < wert2.longValue();
		} else {
			vergleich = wert1.doubleValue() < wert2.doubleValue();
		}

		return LogischerWert.valueOf(vergleich);
	}

	/**
	 * Die Funktion vergleicht zwei Zahlenwerte miteinander. Es wird "wahr"
	 * geliefert, wenn der erste Wert eine kleiner oder gleich große Zahl
	 * repräsentiert wie der zweite.
	 *
	 * @param wert1
	 *            der erste Wert
	 * @param wert2
	 *            der zweite Wert
	 * @return das vergleichsergebnis
	 */
	private LogischerWert kleinerGleich(final Number wert1, final Number wert2) {
		boolean vergleich = true;
		if ((wert1 instanceof Long) && (wert2 instanceof Long)) {
			vergleich = wert1.longValue() <= wert2.longValue();
		} else {
			vergleich = wert1.doubleValue() <= wert2.doubleValue();
		}

		return LogischerWert.valueOf(vergleich);
	}

	/**
	 * Die Funktion liefert den Wert des natürlichen Logarithmus für den
	 * übergebenen Wert.<br>
	 * Das Ergebnis ist immer eine Fliesskommazahl.
	 *
	 * @param logarithmand
	 *            der Logarithmand
	 * @return der Wert des natürlichen Logarithmus
	 */
	private Double ln(final Number logarithmand) {
		return Double.valueOf(Math.log(logarithmand.doubleValue()));
	}

	/**
	 * Die Funktion ermittelt dem maximalen Wert aus einer Liste von
	 * übergebenen.<br>
	 * Das Ergebnis entspricht dem Element mit dem größten Wert.
	 *
	 * @param operanden
	 *            die Liste der zu vergleichenden Werte
	 * @return der größte Wert
	 */
	private Number max(final List<Object> operanden) {
		Number result = null;
		for (final Object obj : operanden) {
			final Number number = (Number) obj;
			if (result == null) {
				result = number;
			} else {
				if (result.doubleValue() < number.doubleValue()) {
					result = number;
				}
			}
		}
		return result;
	}

	/**
	 * Die Funktion ermittelt dem minimalen Wert aus einer Liste von
	 * übergebenen.<br>
	 * Das Ergebnis entspricht dem Element mit dem kleinsten Wert.
	 *
	 * @param operanden
	 *            die Liste der zu vergleichenden Werte
	 * @return der kleinste Wert
	 */
	private Number min(final List<Object> operanden) {
		Number result = null;
		for (final Object obj : operanden) {
			final Number number = (Number) obj;
			if (result == null) {
				result = number;
			} else {
				if (result.doubleValue() > number.doubleValue()) {
					result = number;
				}
			}
		}
		return result;
	}

	/**
	 * die Funktion multipliziert zwei Zahlen. Das Ergebnis ist eine Ganzzahl,
	 * wenn beide Faktoren Ganzzahlen sind, in allen anderen Fällen liefert die
	 * Funktion eine Fließkommazahl als Ergebnis.
	 *
	 * @param faktor1
	 *            der erste Faktor
	 * @param faktor2
	 *            der zweite Faktor
	 * @return das produkt
	 */
	private Number multiplikation(final Number faktor1, final Number faktor2) {
		Number produkt = faktor1;

		if (produkt instanceof Long) {
			if (faktor2 instanceof Long) {
				produkt = produkt.longValue() * faktor2.longValue();
			} else {
				produkt = produkt.longValue() * faktor2.doubleValue();
			}
		} else if (produkt instanceof Double) {
			if (faktor2 instanceof Long) {
				produkt = produkt.doubleValue() * faktor2.longValue();
			} else {
				produkt = produkt.doubleValue() * faktor2.doubleValue();
			}
		}

		return produkt;
	}

	/**
	 * die Funktion negiert einen numerischen Wert, das Typ des Ergebnisses
	 * hängt von dem übergebenen Operanden ab.
	 *
	 * @param operand
	 *            der zu negierende Wert
	 * @return der negierte Wert
	 */
	private Number negativerWert(final Number operand) {
		Number result = null;
		if (operand instanceof Double) {
			result = 0.0 - operand.doubleValue();
		} else {
			result = 0 - operand.longValue();
		}
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#perform(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public Object perform(final Operator operator, final List<Object> operanden) {
		erzeugeHandlerFehler(operator, operanden);

		Object result = null;

		try {
			final Number operand1 = geNumberOperand(operanden, 0);
			final Number operand2 = geNumberOperand(operanden, 1);

			if (ArithmetikOperators.ADDITION.equals(operator)) {
				result = addition(operand1, operand2);
			} else if (ArithmetikOperators.SUBTRAKTION.equals(operator)) {
				if (operanden.size() == 1) {
					result = negativerWert(operand1);
				} else {
					result = subtraktion(operand1, operand2);
				}
			} else if (ArithmetikOperators.MULTIPLIKATION.equals(operator)) {
				result = multiplikation(operand1, operand2);
			} else if (ArithmetikOperators.DIVISION.equals(operator)) {
				result = division(operand1, operand2);
			} else if (ArithmetikOperators.GLEICH.equals(operator)) {
				result = gleich(operand1, operand2);
			} else if (ArithmetikOperators.UNGLEICH.equals(operator)) {
				result = ungleich(operand1, operand2);
			} else if (ArithmetikOperators.GROESSER.equals(operator)) {
				result = groesser(operand1, operand2);
			} else if (ArithmetikOperators.GROESSER_GLEICH.equals(operator)) {
				result = groesserGleich(operand1, operand2);
			} else if (ArithmetikOperators.KLEINER.equals(operator)) {
				result = kleiner(operand1, operand2);
			} else if (ArithmetikOperators.KLEINER_GLEICH.equals(operator)) {
				result = kleinerGleich(operand1, operand2);
			} else if (ArithmetikOperators.GANZZAHL_DIVISION.equals(operator)) {
				result = ganzzahlDivision(operand1, operand2);
			} else if (ArithmetikOperators.GANZZAHL_REST.equals(operator)) {
				result = ganzzahlRest(operand1, operand2);
			} else if (ArithmetikOperators.POTENZ.equals(operator)) {
				result = potenz(operand1, operand2);
			} else if (ArithmetikOperators.EXP.equals(operator)) {
				result = exp(operand1);
			} else if (ArithmetikOperators.LN.equals(operator)) {
				result = ln(operand1);
			} else if (ArithmetikOperators.SIN.equals(operator)) {
				result = sin(operand1);
			} else if (ArithmetikOperators.COS.equals(operator)) {
				result = cos(operand1);
			} else if (ArithmetikOperators.TAN.equals(operator)) {
				result = tan(operand1);
			} else if (ArithmetikOperators.MIN.equals(operator)) {
				result = min(operanden);
			} else if (ArithmetikOperators.MAX.equals(operator)) {
				result = max(operanden);
			} else if (ArithmetikOperators.ABS.equals(operator)) {
				result = abs(operand1);
			} else if (ArithmetikOperators.IN_BOGENMASS.equals(operator)) {
				result = inBogenmass(operand1);
			} else if (ArithmetikOperators.IN_GRAD.equals(operator)) {
				result = inGrad(operand1);
			} else if (ArithmetikOperators.WURZEL.equals(operator)) {
				result = wurzel(operand1, operand2);
			}

		} catch (@SuppressWarnings("unused") final ClassCastException | NullPointerException e) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
		}

		return result;
	}

	/**
	 * Die Funktion liefert die Potenz für die übergebenen Werte von Basis und
	 * Exponent.<br>
	 * Das Ergebnis ist immer eine Fliesskommazahl.
	 *
	 * @param basis
	 *            der Basiswert
	 * @param exponent
	 *            der Exponent
	 * @return der Wert des Sinus
	 */
	private Double potenz(final Number basis, final Number exponent) {
		return Double.valueOf(Math.pow(basis.doubleValue(), exponent.doubleValue()));
	}

	/**
	 * Die Funktion liefert den Sinuswert für den übergebenen Winkel im Gradmaß.
	 * <br>
	 * Das Ergebnis ist immer eine Fliesskommazahl.
	 *
	 * @param winkel
	 *            der Winkel
	 * @return der Wert des Sinus
	 */
	private Double sin(final Number winkel) {
		return Double.valueOf(Math.sin(Math.toRadians(winkel.doubleValue())));
	}

	/**
	 * die Funktion liefert die Differenz zweier Zahlenwerte. Das Ergebnis ist
	 * eine Ganzzahl, wenn beide Minuend und Subtrahend eine Ganzzahl sind. In
	 * allen anderen Fällen ist das Ergebnis eine Fließkommazahl.
	 *
	 * @param minuend
	 *            der Minuend
	 * @param subtrahend
	 *            der Subtrahend
	 * @return die Differenz
	 */
	private Number subtraktion(final Number minuend, final Number subtrahend) {
		Number differenz = minuend;

		if (differenz instanceof Long) {
			if (subtrahend instanceof Long) {
				differenz = differenz.longValue() - subtrahend.longValue();
			} else {
				differenz = differenz.longValue() - subtrahend.doubleValue();
			}
		} else if (differenz instanceof Double) {
			if (subtrahend instanceof Long) {
				differenz = differenz.doubleValue() - subtrahend.longValue();
			} else {
				differenz = differenz.doubleValue() - subtrahend.doubleValue();
			}
		}

		return differenz;
	}

	/**
	 * Die Funktion liefert den Tangenswert für den übergebenen Winkel im
	 * Gradmaß.<br>
	 * Das Ergebnis ist immer eine Fliesskommazahl.
	 *
	 * @param winkel
	 *            der Winkel
	 * @return der Wert des Tangens
	 */
	private Double tan(final Number winkel) {
		return Double.valueOf(Math.tan(Math.toRadians(winkel.doubleValue())));
	}

	/**
	 * Die Funktion vergleicht zwei Zahlenwerte miteinander. Es wird "wahr"
	 * geliefert, wenn beide Zahlen unterschiedliche Werte repräsentieren.
	 *
	 * @param wert1
	 *            der erste Wert
	 * @param wert2
	 *            der zweite Wert
	 * @return das vergleichsergebnis
	 */
	private LogischerWert ungleich(final Number wert1, final Number wert2) {
		boolean vergleich = true;
		if ((wert1 instanceof Long) || (wert2 instanceof Long)) {
			vergleich = wert1.longValue() != wert2.longValue();
		} else {
			vergleich = wert1.doubleValue() != wert2.doubleValue();
		}

		return LogischerWert.valueOf(vergleich);
	}

	/**
	 * Die Funktion ermittelt, ob die übergebenen Operanden mit dem gewünschten
	 * Operator verarbeitet werden können. Für die Ausführung der arithmetischen
	 * Operationen werden als Operanden nur Instanzen vom Typ Number bzw.
	 * DavDatenObjekt (für Daten vom Datenverteiler), die einen Zahlenwert
	 * liefern können erwartet. Die Anzahl der Operanden muss 2 sein, außer für
	 * die Darstellung eines negativen Vorzeichens, welches durch eine
	 * Subtraktionsoperation nachgebildet wird.
	 *
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#validiereHandler(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public HandlerValidation validiereHandler(final Operator operator, final List<? extends Object> operanden) {
		boolean anzahlOk = false;
		boolean typOk = true;

		if (operanden.size() > 0) {
			if (ArithmetikOperators.MIN.equals(operator)) {
				anzahlOk = true;
			} else if (ArithmetikOperators.MAX.equals(operator)) {
				anzahlOk = true;
			}
		}

		if (!anzahlOk) {
			switch (operanden.size()) {
			case 2:
				if (ArithmetikOperators.MULTIPLIKATION.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.DIVISION.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.GANZZAHL_DIVISION.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.GANZZAHL_REST.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.ADDITION.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.SUBTRAKTION.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.GROESSER.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.KLEINER.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.GLEICH.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.UNGLEICH.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.GROESSER_GLEICH.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.KLEINER_GLEICH.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.POTENZ.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.WURZEL.equals(operator)) {
					anzahlOk = true;
				}
				break;
			case 1:
				if (ArithmetikOperators.SUBTRAKTION.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.EXP.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.LN.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.SIN.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.COS.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.TAN.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.ABS.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.IN_BOGENMASS.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.IN_GRAD.equals(operator)) {
					anzahlOk = true;
				} else if (ArithmetikOperators.WURZEL.equals(operator)) {
					anzahlOk = true;
				}
				break;
			default:
				break;
			}
		}

		/* Prüfe, ob alle Argumente Zahlen sind */
		for (final Object operand : operanden) {
			if (!(operand instanceof Number)) {
				typOk = false;
				break;
			}
		}

		return new HandlerValidation(anzahlOk, typOk);
	}

	/**
	 * Die Funktion liefert die Wurzel für die übergebenen Werte von Basis und
	 * Exponent. Wird als Exponent <code>null</code> übergeben, wird der Wert 2
	 * angenommen. Das Ergebnis ist immer eine Fliesskommazahl.
	 *
	 * @param basis
	 *            der Basiswert
	 * @param exponent
	 *            der Exponent
	 * @return der Wert des Sinus
	 */
	private Double wurzel(final Number basis, final Number exponent) {
		Number exp = exponent;
		if (exp == null) {
			exp = 2.0;
		}
		return Double.valueOf(Math.pow(basis.doubleValue(), 1.0 / exp.doubleValue()));
	}
}
