/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.container;

import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import de.bsvrz.ibv.uda.interpreter.Kopierbar;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ContainerZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;

/**
 * Repräsentation einer Schlüsselmenge als spezielle Ausprägung eines
 * {@link UdaContainer}.<br>
 * Als Schlüssel für Uda-Schlüsselmenge sind ausschließlich Zeichenketten
 * erlaubt.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class SchluesselMenge extends UdaContainer implements Kopierbar {

	/**
	 * die Elemente der Menge.
	 */
	private final Map<String, Object> map = new TreeMap<>();

	/**
	 * Standardkonstruktor.<br>
	 * Es wird eine leere Schlüsselmenge erzeugt.
	 */
	public SchluesselMenge() {
		super();
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#elementEinfuegen(int,
	 *      java.lang.Object)
	 */
	@Override
	void elementEinfuegen(final int index, final Object daten) {
		throw new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#elementEinfuegen(java.lang.String,
	 *      java.lang.Object)
	 */
	@Override
	void elementEinfuegen(final String schluessel, final Object daten) {
		if (map.containsKey(schluessel)) {
			throw new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX, "Objekt mit dem Schlüssel: " + schluessel
					+ " ist bereits in der Menge enthalten");
		}
		map.put(schluessel, daten);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#elementLoeschen(int)
	 */
	@Override
	Object elementLoeschen(final int index) {
		return new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#elementLoeschen(java.lang.String)
	 */
	@Override
	Object elementLoeschen(final String schluessel) {
		final Object ergebnis = getContainerElement(schluessel);
		map.remove(schluessel);
		return ergebnis;
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.Kopierbar#erzeugeKopie()
	 */
	@Override
	public Kopierbar erzeugeKopie() {
		final SchluesselMenge result = new SchluesselMenge();
		result.map.putAll(map);
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.Container#getContainerElement(int)
	 * @see #setContainerElement(int, Object)
	 */
	@Override
	public Object getContainerElement(final int index) {
		try {
			final String key = (String) map.keySet().toArray()[index - 1];
			return getContainerElement(key);
		} catch (@SuppressWarnings("unused") final ArrayIndexOutOfBoundsException e) {
			return new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX);
		}
	}

	/**
	 * liefert das Containerelement, das über den übergebenen Schlüssel
	 * identifiziert wird.
	 *
	 * @param schluessel
	 *            der Schlüssel
	 * @return das Element
	 * @see #setContainerElement(String, Object)
	 */
	public Object getContainerElement(final String schluessel) {
		Object result = null;
		result = map.get(schluessel);
		if (result == null) {
			result = new ContainerZugriffsFehler(UdaFehlerSubtyp.SELEKTION);
		}
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#getElemente()
	 */
	@Override
	public Collection<Object> getElemente() {
		return map.values();
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#getGroesse()
	 */
	@Override
	public int getGroesse() {
		return map.size();
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#getSchluessel(int)
	 */
	@Override
	String getSchluessel(final int position) {
		String ergebnis = null;
		if (position > 0) {
			final Set<String> schluessel = map.keySet();
			if (schluessel.size() >= position) {
				ergebnis = schluessel.toArray(new String[0])[position - 1];
			}
		}
		return ergebnis;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer#getSchluesselPosition(java.lang.Object)
	 */
	@Override
	int getSchluesselPosition(final Object schluessel) {
		int index = 1;
		if (schluessel instanceof Number) {
			return ((Number) schluessel).intValue();
		}
		final String search = schluessel.toString();
		for (final String eintrag : map.keySet()) {
			if (search.equals(eintrag)) {
				return index;
			}
			index++;
		}
		throw new ContainerZugriffsFehler(UdaFehlerSubtyp.SELEKTION);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.Container#setContainerElement(int,
	 *      java.lang.Object)
	 * @see #getContainerElement(int)
	 */
	@Override
	public void setContainerElement(final int index, final Object daten) {

		int zugriffsIndex = index;
		if (zugriffsIndex < 0) {
			zugriffsIndex = getGroesse() + 1 + zugriffsIndex;
		}
		if ((zugriffsIndex <= 0) || (zugriffsIndex > (getGroesse() + 1))) {
			throw new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX);
		}

		final String key = map.keySet().toArray(new String[0])[zugriffsIndex - 1];
		if (key != null) {
			map.put(key, daten);
		} else {
			throw new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX);
		}
	}

	/**
	 * setzt die Daten des Containerelements, das über den übergebenen Schlüssel
	 * identifiziert wird.
	 *
	 * @param schluessel
	 *            der Schlüssel
	 * @param daten
	 *            die Daten, die gesetzt werden sollen
	 * @see #getContainerElement(String)
	 */
	public void setContainerElement(final String schluessel, final Object daten) {
		map.put(schluessel, daten);
	}
}
