/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.java;

import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AbstractUdaHandler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.HandlerValidation;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * Handler für Operationen und Funktionen bei der Verwendung eingebundener
 * Javaklassen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class JavaHandler extends AbstractUdaHandler {

	/**
	 * {@inheritDoc}.
	 *
	 * @return die Liste der unterstützten Operatoren
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#getHandledOperators()
	 */
	@Override
	public Operator[] getHandledOperators() {
		return JavaOperators.OPERATOREN;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#validiereHandler(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public HandlerValidation validiereHandler(final Operator operator, final List<? extends Object> operanden) {
		boolean anzahlOk = false;
		boolean typOk = false;
		if (JavaOperators.JAVA_KONSTRUKTOR.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				if (operanden.get(0) instanceof String) {
					typOk = true;
				}
			}
		}

		return new HandlerValidation(anzahlOk, typOk);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#perform(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public Object perform(final Operator operator, final List<Object> operanden) {
		erzeugeHandlerFehler(operator, operanden);
		Object result = null;
		if (JavaOperators.JAVA_KONSTRUKTOR.equals(operator)) {
			try {
				final String name = (String) operanden.get(0);
				result = new JavaObjekt(
						Class.forName(name).getConstructor((Class<?>[]) null).newInstance((Object[]) null));
			} catch (final IllegalArgumentException e) {
				throw new InterpreterException(e);
			} catch (final SecurityException e) {
				throw new InterpreterException(e);
			} catch (final InstantiationException e) {
				throw new InterpreterException(e);
			} catch (final IllegalAccessException e) {
				throw new InterpreterException(e);
			} catch (final InvocationTargetException e) {
				throw new InterpreterException(e);
			} catch (final NoSuchMethodException e) {
				throw new InterpreterException(e);
			} catch (final ClassNotFoundException e) {
				throw new InterpreterException(e);
			}
		}
		return result;
	}

	/**
	 * ermittelt, ob ein Objekt in ein Objekt der übergebenen Typs konvertierbar
	 * ist.
	 *
	 * @param type
	 *            der Zieltyp
	 * @param objekt
	 *            das Objekt, das konvertiert werden soll
	 * @return wahr, wenn das übergebene Objekt in den gegebenen Typ
	 *         konvertierbar ist
	 */
	public static boolean konvertierbar(final Class<?> type, final Object objekt) {
		boolean result = false;
		final String[] numberStrings = { "byte", "Byte", "short", "Short", "int", "Integer", "long", "Long", "float",
				"Float", "double", "Double" };
		if (type.isInstance(objekt)) {
			result = true;
		} else if (objekt instanceof Number) {
			if (Arrays.asList(numberStrings).contains(type.getName())) {
				result = true;
			}
		} else if (objekt instanceof LogischerWert) {
			if ("boolean".equals(type.getName().toLowerCase())) {
				result = true;
			}
		}
		return result;
	}

	/**
	 * konvertiert das übergebene Objekt in Objekt des übergebenen Typs.
	 *
	 * @param typ
	 *            der Zieltyp
	 * @param objekt
	 *            das Objekt, das konvertiert werden soll
	 * @return den konvertierten Wert
	 */
	public static Object konvertiere(final Class<?> typ, final Object objekt) {
		Object result = null;
		final String name = typ.getName();
		if (typ.isInstance(objekt)) {
			result = objekt;
		} else if (objekt instanceof Number) {
			if ("byte".equals(name) || "Byte".equals(name)) {
				result = ((Number) objekt).byteValue();
			} else if ("short".equals(name) || "Short".equals(name)) {
				result = ((Number) objekt).shortValue();
			} else if ("int".equals(name) || "Integer".equals(name)) {
				result = ((Number) objekt).intValue();
			} else if ("long".equals(name) || "Long".equals(name)) {
				result = ((Number) objekt).longValue();
			} else if ("float".equals(name) || "Float".equals(name)) {
				result = ((Number) objekt).floatValue();
			} else if ("double".equals(name.toLowerCase())) {
				result = ((Number) objekt).doubleValue();
			}
		} else if ("boolean".equals(typ.getName().toLowerCase())) {
			if (objekt instanceof LogischerWert) {
				result = ((LogischerWert) objekt).getBoolWert();
			}
		}

		return result;
	}
}
