/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.uda.data;

import java.io.Serializable;

/**
 * die Repräsenation der Daten für die zyklische oder zeitgesteuerte Ausführung
 * eines Uda-Skriptlaufs.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class StartZyklus implements Serializable {
	/**
	 * Versions-ID für die Serialisierung des Objekts.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * der Startzeitpunkt in Millisekunden seit dem 1.1.1970 0 Uhr GMT.
	 */
	private long startZeitPunkt = 1L;

	/**
	 * das Intervall für eine zyklische Ausführung in Millisekunden.
	 */
	private long intervall;

	/**
	 * erzeugt einen Startzyklus mit dem übergebenen Startzeitpunkt und dem
	 * angegebenen Intervall.
	 *
	 * @param startZeit
	 *            der Startzeitpunkt in Millisekunden seit dem 1.1.1970 0 Uhr
	 *            GMT
	 * @param intervall
	 *            das Intervall in Millisekunden
	 */
	public StartZyklus(final long startZeit, final long intervall) {
		setStartZeitPunkt(startZeit);
		setIntervall(intervall);
	}

	/**
	 * liefert das Intervall für die zyklische Ausführung eines Skriptlaufs in
	 * Millisekunden.
	 *
	 * @return das Intervall
	 * @see #setIntervall(long)
	 */
	public long getIntervall() {
		return intervall;
	}

	/**
	 * setzt das Intervall für die zyklische Ausführung eines Uda-Skriptlaufs.
	 * Der Wert wird in Millisekunden angegeben.
	 *
	 * @param intervall
	 *            das Intervall
	 * @see #getIntervall()
	 */
	public void setIntervall(final long intervall) {
		this.intervall = intervall;
	}

	/**
	 * liefert den Startzeitpunkt für die Ausführung eines Skriptlaufs in
	 * Millisekunden seit dem 1.1.1970 0 Uhr GMT.
	 *
	 * @return den Zeitpunkt
	 * @see #setStartZeitPunkt(long)
	 */
	public long getStartZeitPunkt() {
		return startZeitPunkt;
	}

	/**
	 * setzt den Startzeitpunkt für die Ausführung eines Skriptlaufs in
	 * Millisekunden seit dem 1.1.1970 0 Uhr GMT.
	 *
	 * @param startZeitPunkt
	 *            der Startzeitpunkt
	 * @see #getStartZeitPunkt()
	 */
	public void setStartZeitPunkt(final long startZeitPunkt) {
		if (startZeitPunkt <= 0) {
			this.startZeitPunkt = 1L;
		} else {
			this.startZeitPunkt = startZeitPunkt;
		}
	}

	/**
	 * liefert den dem Zyklus entsprechenden nächstmöglichen Startzeitpunkt
	 * ausgehend vom übergebenen Zeitpunkt.
	 *
	 * @param letzterStartZeitPunkt
	 *            in Millisekunden seit dem 1.1.1970 0 Uhr GMT
	 * @return den Zeitpunkt
	 */
	public long getNaechstenStartZeitPunkt(final long letzterStartZeitPunkt) {
		long ergebnis = startZeitPunkt;
		if ((intervall > 0) && (letzterStartZeitPunkt > startZeitPunkt)) {
			final long intervalle = ((letzterStartZeitPunkt - startZeitPunkt) / intervall) + 1;
			ergebnis += (intervalle * intervall);
		}
		return ergebnis;
	}
}
